# This file was autogenerated using update.py on 2026-02-17 17:48:24.761158

TEST_CERTS = {"Test ESTEID2025": "testESTEID2025.pem"}
LIVE_CERTS = {"ESTEID2025": "ESTEID2025.pem"}
ROOT_CA_FILE_NAME = "EEGovCA2025.pem"
TEST_ROOT_CA_FILE_NAME = "testEEGovCA2025.pem"
